#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# This file incorporates work covered by the following license notice:
#
#   Licensed to the Apache Software Foundation (ASF) under one or more
#   contributor license agreements. See the NOTICE file distributed
#   with this work for additional information regarding copyright
#   ownership. The ASF licenses this file to you under the Apache
#   License, Version 2.0 (the "License"); you may not use this file
#   except in compliance with the License. You may obtain a copy of
#   the License at http://www.apache.org/licenses/LICENSE-2.0 .
#
HID0_WEBWIZARD = 34200 + 0
HID0_WEBWIZARD = 34200 + 0
HID0_HELP = 34200 + 1
HID0_NEXT = 34200 + 2
HID0_PREV = 34200 + 3
HID0_CREATE = 34200 + 4
HID0_CANCEL = 34200 + 5
HID0_STATUS_DIALOG = 34200 + 6  #step 1
HID1_LST_SESSIONS = 34200 + 7
HID1_BTN_DEL_SES = 34200 + 9    #step 2
HID2_LST_DOCS = 34200 + 10
HID2_BTN_ADD_DOC = 34200 + 11
HID2_BTN_REM_DOC = 34200 + 12
HID2_BTN_DOC_UP = 34200 + 13
HID2_BTN_DOC_DOWN = 34200 + 14
HID2_TXT_DOC_TITLE = 34200 + 15
HID2_TXT_DOC_DESC = 34200 + 16
HID2_TXT_DOC_AUTHOR = 34200 + 17
HID2_LST_DOC_EXPORT = 34200 + 18
HID2_STATUS_ADD_DOCS = 34200 + 19 #step 3
HID3_IL_LAYOUTS_IMG1 = 34200 + 20
HID3_IL_LAYOUTS_IMG2 = 34200 + 21
HID3_IL_LAYOUTS_IMG3 = 34200 + 22
HID3_IL_LAYOUTS_IMG4 = 34200 + 23
HID3_IL_LAYOUTS_IMG5 = 34200 + 24
HID3_IL_LAYOUTS_IMG6 = 34200 + 25
HID3_IL_LAYOUTS_IMG7 = 34200 + 26
HID3_IL_LAYOUTS_IMG8 = 34200 + 27
HID3_IL_LAYOUTS_IMG9 = 34200 + 28
HID3_IL_LAYOUTS_IMG10 = 34200 + 29
HID3_IL_LAYOUTS_IMG11 = 34200 + 30
HID3_IL_LAYOUTS_IMG12 = 34200 + 31
HID3_IL_LAYOUTS_IMG13 = 34200 + 32
HID3_IL_LAYOUTS_IMG14 = 34200 + 33
HID3_IL_LAYOUTS_IMG15 = 34200 + 34  #step 4
HID4_CHK_DISPLAY_FILENAME = 34200 + 35
HID4_CHK_DISPLAY_DESCRIPTION = 34200 + 36
HID4_CHK_DISPLAY_AUTHOR = 34200 + 37
HID4_CHK_DISPLAY_CR_DATE = 34200 + 38
HID4_CHK_DISPLAY_UP_DATE = 34200 + 39
HID4_CHK_DISPLAY_FORMAT = 34200 + 40
HID4_CHK_DISPLAY_F_ICON = 34200 + 41
HID4_CHK_DISPLAY_PAGES = 34200 + 42
HID4_CHK_DISPLAY_SIZE = 34200 + 43
HID4_GRP_OPTIMAIZE_640 = 34200 + 44
HID4_GRP_OPTIMAIZE_800 = 34200 + 45
HID4_GRP_OPTIMAIZE_1024 = 34200 + 46    #step 5
HID5_LST_STYLES = 34200 + 47
HID5_BTN_BACKGND = 34200 + 48
HID5_BTN_ICONS = 34200 + 49 #step 6
HID6_TXT_SITE_TITLE = 34200 + 50
HID6_TXT_SITE_ICON = 34200 + 51
HID6_BTN_SITE_ICON = 34200 + 52
HID6_TXT_SITE_DESC = 34200 + 53
HID6_TXT_SITE_KEYWRDS = 34200 + 54
HID6_DATE_SITE_CREATED = 34200 + 55
HID6_DATE_SITE_UPDATED = 34200 + 56
HID6_NUM_SITE_REVISTS = 34200 + 57
HID6_TXT_SITE_EMAIL = 34200 + 58
HID6_TXT_SITE_COPYRIGHT = 34200 + 59    #step 7
HID7_BTN_PREVIEW = 34200 + 60
HID7_CHK_PUBLISH_LOCAL = 34200 + 61
HID7_TXT_LOCAL = 34200 + 62
HID7_BTN_LOCAL = 34200 + 63
HID7_CHK_PUBLISH_ZIP = 34200 + 64
HID7_TXT_ZIP = 34200 + 65
HID7_BTN_ZIP = 34200 + 66
HID7_CHK_PUBLISH_FTP = 34200 + 67
HID7_BTN_FTP = 34200 + 69
HID7_CHK_SAVE = 34200 + 70
HID7_TXT_SAVE = 34200 + 71  #web wizard backgrounds dialog
HID_BG = 34200 + 90
HID_BG_BTN_OTHER = 34200 + 91
HID_BG_BTN_NONE = 34200 + 92
HID_BG_BTN_OK = 34200 + 93
HID_BG_BTN_CANCEL = 34200 + 94
HID_BG_BTN_BACK = 34200 + 95
HID_BG_BTN_FW = 34200 + 96
HID_BG_BTN_IMG1 = 34200 + 97
HID_BG_BTN_IMG2 = 34200 + 98
HID_BG_BTN_IMG3 = 34200 + 99
HID_BG_BTN_IMG4 = 34200 + 100
HID_BG_BTN_IMG5 = 34200 + 101
HID_BG_BTN_IMG6 = 34200 + 102
HID_BG_BTN_IMG7 = 34200 + 103
HID_BG_BTN_IMG8 = 34200 + 104
HID_BG_BTN_IMG9 = 34200 + 105
HID_BG_BTN_IMG10 = 34200 + 106
HID_BG_BTN_IMG11 = 34200 + 107
HID_BG_BTN_IMG12 = 34200 + 108  #web wizard icons sets dialog
HID_IS = 41000 + 0
HID_IS_ICONSETS = 41000 + 1
HID_IS_BTN_NONE = 41000 + 2
HID_IS_BTN_OK = 41000 + 3
HID_IS_BTN_IMG1 = 41000 + 5
HID_IS_BTN_IMG2 = 41000 + 6
HID_IS_BTN_IMG3 = 41000 + 7
HID_IS_BTN_IMG4 = 41000 + 8
HID_IS_BTN_IMG5 = 41000 + 9
HID_IS_BTN_IMG6 = 41000 + 10
HID_IS_BTN_IMG7 = 41000 + 11
HID_IS_BTN_IMG8 = 41000 + 12
HID_IS_BTN_IMG9 = 41000 + 13
HID_IS_BTN_IMG10 = 41000 + 14
HID_IS_BTN_IMG11 = 41000 + 15
HID_IS_BTN_IMG12 = 41000 + 16
HID_IS_BTN_IMG13 = 41000 + 17
HID_IS_BTN_IMG14 = 41000 + 18
HID_IS_BTN_IMG15 = 41000 + 19
HID_IS_BTN_IMG16 = 41000 + 20
HID_IS_BTN_IMG17 = 41000 + 21
HID_IS_BTN_IMG18 = 41000 + 22
HID_IS_BTN_IMG19 = 41000 + 23
HID_IS_BTN_IMG20 = 41000 + 24
HID_IS_BTN_IMG21 = 41000 + 25
HID_IS_BTN_IMG22 = 41000 + 26
HID_IS_BTN_IMG23 = 41000 + 27
HID_IS_BTN_IMG24 = 41000 + 28
HID_IS_BTN_IMG25 = 41000 + 29
HID_IS_BTN_IMG26 = 41000 + 30
HID_IS_BTN_IMG27 = 41000 + 31
HID_IS_BTN_IMG28 = 41000 + 32
HID_IS_BTN_IMG29 = 41000 + 33
HID_IS_BTN_IMG30 = 41000 + 34
HID_IS_BTN_IMG31 = 41000 + 35
HID_IS_BTN_IMG32 = 41000 + 36

#   web wizard ftp dialog
HID_FTP = 41000 + 40
HID_FTP_SERVER = 41000 + 41
HID_FTP_USERNAME = 41000 + 42
HID_FTP_PASS = 41000 + 43
HID_FTP_TEST = 41000 + 44
HID_FTP_TXT_PATH = 41000 + 45
HID_FTP_BTN_PATH = 41000 + 46
HID_FTP_OK = 41000 + 47
HID_FTP_CANCEL = 41000 + 48
