/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
package com.sun.star.wizards.common;

public class HelpIds
{
    private static String array1[] = new String[]
    {
        "HID:WIZARDS_HID0_WEBWIZARD",    // HID:34200
        "HID:WIZARDS_HID0_HELP",    // HID:34201
        "HID:WIZARDS_HID0_NEXT",    // HID:34202
        "HID:WIZARDS_HID0_PREV",    // HID:34203
        "HID:WIZARDS_HID0_CREATE",    // HID:34204
        "HID:WIZARDS_HID0_CANCEL",    // HID:34205
        "HID:WIZARDS_HID0_STATUS_DIALOG",    // HID:34206
        "HID:WIZARDS_HID1_LST_SESSIONS",    // HID:34207
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID1_BTN_DEL_SES",    // HID:34209
        "HID:WIZARDS_HID2_LST_DOCS",    // HID:34210
        "HID:WIZARDS_HID2_BTN_ADD_DOC",    // HID:34211
        "HID:WIZARDS_HID2_BTN_REM_DOC",    // HID:34212
        "HID:WIZARDS_HID2_BTN_DOC_UP",    // HID:34213
        "HID:WIZARDS_HID2_BTN_DOC_DOWN",    // HID:34214
        "HID:WIZARDS_HID2_TXT_DOC_TITLE",    // HID:34215
        "HID:WIZARDS_HID2_TXT_DOC_DESC",    // HID:34216
        "HID:WIZARDS_HID2_TXT_DOC_AUTHOR",    // HID:34217
        "HID:WIZARDS_HID2_LST_DOC_EXPORT",    // HID:34218
        "HID:WIZARDS_HID2_STATUS_ADD_DOCS",    // HID:34219
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG1",    // HID:34220
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG2",    // HID:34221
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG3",    // HID:34222
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG4",    // HID:34223
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG5",    // HID:34224
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG6",    // HID:34225
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG7",    // HID:34226
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG8",    // HID:34227
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG9",    // HID:34228
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG10",    // HID:34229
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG11",    // HID:34230
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG12",    // HID:34231
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG13",    // HID:34232
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG14",    // HID:34233
        "HID:WIZARDS_HID3_IL_LAYOUTS_IMG15",    // HID:34234
        "HID:WIZARDS_HID4_CHK_DISPLAY_FILENAME",    // HID:34235
        "HID:WIZARDS_HID4_CHK_DISPLAY_DESCRIPTION",    // HID:34236
        "HID:WIZARDS_HID4_CHK_DISPLAY_AUTHOR",    // HID:34237
        "HID:WIZARDS_HID4_CHK_DISPLAY_CR_DATE",    // HID:34238
        "HID:WIZARDS_HID4_CHK_DISPLAY_UP_DATE",    // HID:34239
        "HID:WIZARDS_HID4_CHK_DISPLAY_FORMAT",    // HID:34240
        "HID:WIZARDS_HID4_CHK_DISPLAY_F_ICON",    // HID:34241
        "HID:WIZARDS_HID4_CHK_DISPLAY_PAGES",    // HID:34242
        "HID:WIZARDS_HID4_CHK_DISPLAY_SIZE",    // HID:34243
        "HID:WIZARDS_HID4_GRP_OPTIMAIZE_640",    // HID:34244
        "HID:WIZARDS_HID4_GRP_OPTIMAIZE_800",    // HID:34245
        "HID:WIZARDS_HID4_GRP_OPTIMAIZE_1024",    // HID:34246
        "HID:WIZARDS_HID5_LST_STYLES",    // HID:34247
        "HID:WIZARDS_HID5_BTN_BACKGND",    // HID:34248
        "HID:WIZARDS_HID5_BTN_ICONS",    // HID:34249
        "HID:WIZARDS_HID6_TXT_SITE_TITLE",    // HID:34250
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID6_TXT_SITE_DESC",    // HID:34253
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID6_DATE_SITE_CREATED",    // HID:34255
        "HID:WIZARDS_HID6_DATE_SITE_UPDATED",    // HID:34256
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID6_TXT_SITE_EMAIL",    // HID:34258
        "HID:WIZARDS_HID6_TXT_SITE_COPYRIGHT",    // HID:34259
        "HID:WIZARDS_HID7_BTN_PREVIEW",    // HID:34260
        "HID:WIZARDS_HID7_CHK_PUBLISH_LOCAL",    // HID:34261
        "HID:WIZARDS_HID7_TXT_LOCAL",    // HID:34262
        "HID:WIZARDS_HID7_BTN_LOCAL",    // HID:34263
        "HID:WIZARDS_HID7_CHK_PUBLISH_ZIP",    // HID:34264
        "HID:WIZARDS_HID7_TXT_ZIP",    // HID:34265
        "HID:WIZARDS_HID7_BTN_ZIP",    // HID:34266
        "HID:WIZARDS_HID7_CHK_PUBLISH_FTP",    // HID:34267
        "HID:WIZARDS_HID7_TXT_FTP",    // HID:34268
        "HID:WIZARDS_HID7_BTN_FTP",    // HID:34269
        "HID:WIZARDS_HID7_CHK_SAVE",    // HID:34270
        "HID:WIZARDS_HID7_TXT_SAVE",    // HID:34271
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_BG",    // HID:34290
        "HID:WIZARDS_HID_BG_BTN_OTHER",    // HID:34291
        "HID:WIZARDS_HID_BG_BTN_NONE",    // HID:34292
        "HID:WIZARDS_HID_BG_BTN_OK",    // HID:34293
        "HID:WIZARDS_HID_BG_BTN_CANCEL",    // HID:34294
        "HID:WIZARDS_HID_BG_BTN_BACK",    // HID:34295
        "HID:WIZARDS_HID_BG_BTN_FW",    // HID:34296
        "HID:WIZARDS_HID_BG_BTN_IMG1",    // HID:34297
        "HID:WIZARDS_HID_BG_BTN_IMG2",    // HID:34298
        "HID:WIZARDS_HID_BG_BTN_IMG3",    // HID:34299
        "HID:WIZARDS_HID_BG_BTN_IMG4",    // HID:34300
        "HID:WIZARDS_HID_BG_BTN_IMG5",    // HID:34301
        "HID:WIZARDS_HID_BG_BTN_IMG6",    // HID:34302
        "HID:WIZARDS_HID_BG_BTN_IMG7",    // HID:34303
        "HID:WIZARDS_HID_BG_BTN_IMG8",    // HID:34304
        "HID:WIZARDS_HID_BG_BTN_IMG9",    // HID:34305
        "HID:WIZARDS_HID_BG_BTN_IMG10",    // HID:34306
        "HID:WIZARDS_HID_BG_BTN_IMG11",    // HID:34307
        "HID:WIZARDS_HID_BG_BTN_IMG12",    // HID:34308
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_DIALOG",    // HID:34320
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_0_CMDPREV",    // HID:34322
        "HID:WIZARDS_HID_DLGREPORT_0_CMDNEXT",    // HID:34323
        "HID:WIZARDS_HID_DLGREPORT_0_CMDFINISH",    // HID:34324
        "HID:WIZARDS_HID_DLGREPORT_0_CMDCANCEL",    // HID:34325
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_1_LBTABLES",    // HID:34330
        "HID:WIZARDS_HID_DLGREPORT_1_FIELDSAVAILABLE",    // HID:34331
        "HID:WIZARDS_HID_DLGREPORT_1_CMDMOVESELECTED",    // HID:34332
        "HID:WIZARDS_HID_DLGREPORT_1_CMDMOVEALL",    // HID:34333
        "HID:WIZARDS_HID_DLGREPORT_1_CMDREMOVESELECTED",    // HID:34334
        "HID:WIZARDS_HID_DLGREPORT_1_CMDREMOVEALL",    // HID:34335
        "HID:WIZARDS_HID_DLGREPORT_1_FIELDSSELECTED",    // HID:34336
        "HID:WIZARDS_HID_DLGREPORT_1_CMDMOVEUP",    // HID:34337
        "HID:WIZARDS_HID_DLGREPORT_1_CMDMOVEDOWN",    // HID:34338
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_2_GROUPING",    // HID:34340
        "HID:WIZARDS_HID_DLGREPORT_2_CMDGROUP",    // HID:34341
        "HID:WIZARDS_HID_DLGREPORT_2_CMDUNGROUP",    // HID:34342
        "HID:WIZARDS_HID_DLGREPORT_2_PREGROUPINGDEST",    // HID:34343
        "HID:WIZARDS_HID_DLGREPORT_2_CMDMOVEUPGROUP",    // HID:34344
        "HID:WIZARDS_HID_DLGREPORT_2_CMDMOVEDOWNGROUP",    // HID:34345
        "HID:WIZARDS_HID_DLGREPORT_3_SORT1",    // HID:34346
        "HID:WIZARDS_HID_DLGREPORT_3_OPTASCEND1",    // HID:34347
        "HID:WIZARDS_HID_DLGREPORT_3_OPTDESCEND1",    // HID:34348
        "HID:WIZARDS_HID_DLGREPORT_3_SORT2",    // HID:34349
        "HID:WIZARDS_HID_DLGREPORT_3_OPTASCEND2",    // HID:34350
        "HID:WIZARDS_HID_DLGREPORT_3_OPTDESCEND2",    // HID:34351
        "HID:WIZARDS_HID_DLGREPORT_3_SORT3",    // HID:34352
        "HID:WIZARDS_HID_DLGREPORT_3_OPTASCEND3",    // HID:34353
        "HID:WIZARDS_HID_DLGREPORT_3_OPTDESCEND3",    // HID:34354
        "HID:WIZARDS_HID_DLGREPORT_3_SORT4",    // HID:34355
        "HID:WIZARDS_HID_DLGREPORT_3_OPTASCEND4",    // HID:34356
        "HID:WIZARDS_HID_DLGREPORT_3_OPTDESCEND4",    // HID:34357
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_4_TITLE",    // HID:34362
        "HID:WIZARDS_HID_DLGREPORT_4_DATALAYOUT",    // HID:34363
        "HID:WIZARDS_HID_DLGREPORT_4_PAGELAYOUT",    // HID:34364
        "HID:WIZARDS_HID_DLGREPORT_4_LANDSCAPE",    // HID:34365
        "HID:WIZARDS_HID_DLGREPORT_4_PORTRAIT",    // HID:34366
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_5_OPTDYNTEMPLATE",    // HID:34370
        "HID:WIZARDS_HID_DLGREPORT_5_OPTSTATDOCUMENT",    // HID:34371
        "HID:WIZARDS_HID_DLGREPORT_5_TXTTEMPLATEPATH",    // HID:34372
        "HID:WIZARDS_HID_DLGREPORT_5_CMDTEMPLATEPATH",    // HID:34373
        "HID:WIZARDS_HID_DLGREPORT_5_OPTEDITTEMPLATE",    // HID:34374
        "HID:WIZARDS_HID_DLGREPORT_5_OPTUSETEMPLATE",    // HID:34375
        "HID:WIZARDS_HID_DLGREPORT_5_TXTDOCUMENTPATH",    // HID:34376
        "HID:WIZARDS_HID_DLGREPORT_5_CMDDOCUMENTPATH",    // HID:34377
        "HID:WIZARDS_HID_DLGREPORT_5_CHKLINKTODB",    // HID:34378
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_1",    // HID:34381
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_2",    // HID:34382
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_3",    // HID:34383
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_4",    // HID:34384
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_5",    // HID:34385
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_6",    // HID:34386
        "HID:WIZARDS_HID_DLGREPORT_6_TXTTITLE_7",    // HID:34387
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_DIALOG",    // HID:34400
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_CMDPREV",    // HID:34402
        "HID:WIZARDS_HID_DLGFORM_CMDNEXT",    // HID:34403
        "HID:WIZARDS_HID_DLGFORM_CMDFINISH",    // HID:34404
        "HID:WIZARDS_HID_DLGFORM_CMDCANCEL",    // HID:34405
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_MASTER_LBTABLES",    // HID:34411
        "HID:WIZARDS_HID_DLGFORM_MASTER_FIELDSAVAILABLE",    // HID:34412
        "HID:WIZARDS_HID_DLGFORM_MASTER_CMDMOVESELECTED",    // HID:34413
        "HID:WIZARDS_HID_DLGFORM_MASTER_CMDMOVEALL",    // HID:34414
        "HID:WIZARDS_HID_DLGFORM_MASTER_CMDREMOVESELECTED",    // HID:34415
        "HID:WIZARDS_HID_DLGFORM_MASTER_CMDREMOVEALL",    // HID:34416
        "HID:WIZARDS_HID_DLGFORM_MASTER_FIELDSSELECTED",    // HID:34417
        "HID:WIZARDS_HID_DLGFORM_MASTER_CMDMOVEUP",    // HID:34418
        "HID:WIZARDS_HID_DLGFORM_MASTER_CMDMOVEDOWN",    // HID:34419
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_CHKCREATESUBFORM",    // HID:34421
        "HID:WIZARDS_HID_DLGFORM_OPTONEXISTINGRELATION",    // HID:34422
        "HID:WIZARDS_HID_DLGFORM_OPTSELECTMANUALLY",    // HID:34423
        "HID:WIZARDS_HID_DLGFORM_lstRELATIONS",    // HID:34424
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_SUB_LBTABLES",    // HID:34431
        "HID:WIZARDS_HID_DLGFORM_SUB_FIELDSAVAILABLE",    // HID:34432
        "HID:WIZARDS_HID_DLGFORM_SUB_CMDMOVESELECTED",    // HID:34433
        "HID:WIZARDS_HID_DLGFORM_SUB_CMDMOVEALL",    // HID:34434
        "HID:WIZARDS_HID_DLGFORM_SUB_CMDREMOVESELECTED",    // HID:34435
        "HID:WIZARDS_HID_DLGFORM_SUB_CMDREMOVEALL",    // HID:34436
        "HID:WIZARDS_HID_DLGFORM_SUB_FIELDSSELECTED",    // HID:34437
        "HID:WIZARDS_HID_DLGFORM_SUB_CMDMOVEUP",    // HID:34438
        "HID:WIZARDS_HID_DLGFORM_SUB_CMDMOVEDOWN",    // HID:34439
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTSLAVELINK1",    // HID:34441
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTMASTERLINK1",    // HID:34442
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTSLAVELINK2",    // HID:34443
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTMASTERLINK2",    // HID:34444
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTSLAVELINK3",    // HID:34445
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTMASTERLINK3",    // HID:34446
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTSLAVELINK4",    // HID:34447
        "HID:WIZARDS_HID_DLGFORM_LINKER_LSTMASTERLINK4",    // HID:34448
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_CMDALIGNLEFT",    // HID:34451
        "HID:WIZARDS_HID_DLGFORM_CMDALIGNRIGHT",    // HID:34452
        "HID:WIZARDS_HID_DLGFORM_CMDLEFTLABELED",    // HID:34453
        "HID:WIZARDS_HID_DLGFORM_CMDTOPLABELED",    // HID:34454
        "HID:WIZARDS_HID_DLGFORM_CMDTABLESTYLE",    // HID:34455
        "HID:WIZARDS_HID_DLGFORM_CMDTOPJUSTIFIED",    // HID:34456
        "HID:WIZARDS_HID_DLGFORM_CMDLEFTLABELED2",    // HID:34457
        "HID:WIZARDS_HID_DLGFORM_CMDTOPLABELED2",    // HID:34458
        "HID:WIZARDS_HID_DLGFORM_CMDTABLESTYLE2",    // HID:34459
        "HID:WIZARDS_HID_DLGFORM_CMDTOPJUSTIFIED2",    // HID:34460
        "HID:WIZARDS_HID_DLGFORM_OPTNEWDATAONLY",    // HID:34461
        "HID:WIZARDS_HID_DLGFORM_OPTDISPLAYALLDATA",    // HID:34462
        "HID:WIZARDS_HID_DLGFORM_CHKNOMODIFICATION",    // HID:34463
        "HID:WIZARDS_HID_DLGFORM_CHKNODELETION",    // HID:34464
        "HID:WIZARDS_HID_DLGFORM_CHKNOADDITION",    // HID:34465
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_LSTSTYLES",    // HID:34471
        "HID:WIZARDS_HID_DLGFORM_CMDNOBORDER",    // HID:34472
        "HID:WIZARDS_HID_DLGFORM_CMD3DBORDER",    // HID:34473
        "HID:WIZARDS_HID_DLGFORM_CMDSIMPLEBORDER",    // HID:34474
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGFORM_TXTPATH",    // HID:34481
        "HID:WIZARDS_HID_DLGFORM_OPTWORKWITHFORM",    // HID:34482
        "HID:WIZARDS_HID_DLGFORM_OPTMODIFYFORM",    // HID:34483
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGNEWSLTR_DIALOG",    // HID:34500
        "HID:WIZARDS_HID_DLGNEWSLTR_OPTSTANDARDLAYOUT",    // HID:34501
        "HID:WIZARDS_HID_DLGNEWSLTR_OPTPARTYLAYOUT",    // HID:34502
        "HID:WIZARDS_HID_DLGNEWSLTR_OPTBROCHURELAYOUT",    // HID:34503
        "HID:WIZARDS_HID_DLGNEWSLTR_OPTSINGLESIDED",    // HID:34504
        "HID:WIZARDS_HID_DLGNEWSLTR_OPTDOUBLESIDED",    // HID:34505
        "HID:WIZARDS_HID_DLGNEWSLTR_CMDGOON",    // HID:34506
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGDEPOT_DIALOG_SELLBUY",    // HID:34520
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTSTOCKID_SELLBUY",    // HID:34521
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTQUANTITY",    // HID:34522
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTRATE",    // HID:34523
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTDATE",    // HID:34524
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTCOMMISSION",    // HID:34525
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTFIX",    // HID:34526
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTMINIMUM",    // HID:34527
        "HID:WIZARDS_HID_DLGDEPOT_0_CMDCANCEL_SELLBUY",    // HID:34528
        "HID:WIZARDS_HID_DLGDEPOT_0_CMDGOON_SELLBUY",    // HID:34529
        "HID:WIZARDS_HID_DLGDEPOT_1_LSTSELLSTOCKS",    // HID:34530
        "HID:WIZARDS_HID_DLGDEPOT_2_LSTBUYSTOCKS",    // HID:34531
        "HID:WIZARDS_HID_DLGDEPOT_DIALOG_SPLIT",    // HID:34532
        "HID:WIZARDS_HID_DLGDEPOT_0_LSTSTOCKNAMES",    // HID:34533
        "HID:WIZARDS_HID_DLGDEPOT_0_TXTSTOCKID_SPLIT",    // HID:34534
        "HID:WIZARDS_HID_DLGDEPOT_0_CMDCANCEL_SPLIT",    // HID:34535
        "HID:WIZARDS_HID_DLGDEPOT_0_CMDGOON_SPLIT",    // HID:34536
        "HID:WIZARDS_HID_DLGDEPOT_1_OPTPERSHARE",    // HID:34537
        "HID:WIZARDS_HID_DLGDEPOT_1_OPTTOTAL",    // HID:34538
        "HID:WIZARDS_HID_DLGDEPOT_1_TXTDIVIDEND",    // HID:34539
        "HID:WIZARDS_HID_DLGDEPOT_2_TXTOLDRATE",    // HID:34540
        "HID:WIZARDS_HID_DLGDEPOT_2_TXTNEWRATE",    // HID:34541
        "HID:WIZARDS_HID_DLGDEPOT_2_TXTDATE",    // HID:34542
        "HID:WIZARDS_HID_DLGDEPOT_3_TXTSTARTDATE",    // HID:34543
        "HID:WIZARDS_HID_DLGDEPOT_3_TXTENDDATE",    // HID:34544
        "HID:WIZARDS_HID_DLGDEPOT_3_OPTDAILY",    // HID:34545
        "HID:WIZARDS_HID_DLGDEPOT_3_OPTWEEKLY",    // HID:34546
        "HID:WIZARDS_HID_DLGDEPOT_DIALOG_HISTORY",    // HID:34547
        "HID:WIZARDS_HID_DLGDEPOT_LSTMARKETS",    // HID:34548
        "HID:WIZARDS_HID_DLGDEPOT_0_CMDCANCEL_HISTORY",    // HID:34549
        "HID:WIZARDS_HID_DLGDEPOT_0_CMDGOON_HISTORY",    // HID:34550
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGIMPORT_DIALOG",    // HID:34570
        "HID:WIZARDS_HID_DLGIMPORT_0_CMDHELP",    // HID:34571
        "HID:WIZARDS_HID_DLGIMPORT_0_CMDCANCEL",    // HID:34572
        "HID:WIZARDS_HID_DLGIMPORT_0_CMDPREV",    // HID:34573
        "HID:WIZARDS_HID_DLGIMPORT_0_CMDNEXT",    // HID:34574
        "HID:WIZARDS_HID_DLGIMPORT_0_OPTSODOCUMENTS",    // HID:34575
        "HID:WIZARDS_HID_DLGIMPORT_0_OPTMSDOCUMENTS",    // HID:34576
        "HID:WIZARDS_HID_DLGIMPORT_0_CHKLOGFILE",    // HID:34577
        "HID:WIZARDS_HID_DLGIMPORT_2_CHKWORD",    // HID:34578
        "HID:WIZARDS_HID_DLGIMPORT_2_CHKEXCEL",    // HID:34579
        "HID:WIZARDS_HID_DLGIMPORT_2_CHKPOWERPOINT",    // HID:34580
        "HID:WIZARDS_HID_DLGIMPORT_2_CBTEMPLATE",    // HID:34581
        "HID:WIZARDS_HID_DLGIMPORT_2_CBTEMPLATERECURSE",    // HID:34582
        "HID:WIZARDS_HID_DLGIMPORT_2_LBTEMPLATEPATH",    // HID:34583
        "HID:WIZARDS_HID_DLGIMPORT_2_EDTEMPLATEPATH",    // HID:34584
        "HID:WIZARDS_HID_DLGIMPORT_2_CMDTEMPLATEPATHSELECT",    // HID:34585
        "HID:WIZARDS_HID_DLGIMPORT_2_CBDOCUMENT",    // HID:34586
        "HID:WIZARDS_HID_DLGIMPORT_2_CBDOCUMENTRECURSE",    // HID:34587
        "HID:WIZARDS_HID_DLGIMPORT_2_LBDOCUMENTPATH",    // HID:34588
        "HID:WIZARDS_HID_DLGIMPORT_2_EDDOCUMENTPATH",    // HID:34589
        "HID:WIZARDS_HID_DLGIMPORT_2_CMDDOCUMENTPATHSELECT",    // HID:34590
        "HID:WIZARDS_HID_DLGIMPORT_2_LBEXPORTDOCUMENTPATH",    // HID:34591
        "HID:WIZARDS_HID_DLGIMPORT_2_EDEXPORTDOCUMENTPATH",    // HID:34592
        "HID:WIZARDS_HID_DLGIMPORT_2_CMDEXPORTPATHSELECT",    // HID:34593
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGIMPORT_3_TBSUMMARY",    // HID:34595
        "HID:WIZARDS_HID_DLGIMPORT_0_CHKWRITER",    // HID:34596
        "HID:WIZARDS_HID_DLGIMPORT_0_CHKCALC",    // HID:34597
        "HID:WIZARDS_HID_DLGIMPORT_0_CHKIMPRESS",    // HID:34598
        "HID:WIZARDS_HID_DLGIMPORT_0_CHKMATHGLOBAL",    // HID:34599
        "HID:WIZARDS_HID_DLGIMPORT_2_CMDTEMPLATEPATHSELECT2",    // HID:34600
        "HID:WIZARDS_HID_DLGIMPORT_2_CMDDOCUMENTPATHSELECT2",    // HID:34601
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_DIALOG",    // HID:34630
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_CANCEL",    // HID:34631
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_OPTIONAGENDA1",    // HID:34632
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_OPTIONAGENDA2",    // HID:34633
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_AGENDAOKAY",    // HID:34634
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_OPTIONLETTER1",    // HID:34635
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_OPTIONLETTER2",    // HID:34636
        "HID:WIZARDS_HID_DLGCORRESPONDENCE_LETTEROKAY",    // HID:34637
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGSTYLES_DIALOG",    // HID:34650
        "HID:WIZARDS_HID_DLGSTYLES_LISTBOX",    // HID:34651
        "HID:WIZARDS_HID_DLGSTYLES_CANCEL",    // HID:34652
        "HID:WIZARDS_HID_DLGSTYLES_OKAY",    // HID:34653
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGCONVERT_DIALOG",    // HID:34660
        "HID:WIZARDS_HID_DLGCONVERT_CHECKBOX1",    // HID:34661
        "HID:WIZARDS_HID_DLGCONVERT_OPTIONBUTTON1",    // HID:34662
        "HID:WIZARDS_HID_DLGCONVERT_OPTIONBUTTON2",    // HID:34663
        "HID:WIZARDS_HID_DLGCONVERT_OPTIONBUTTON3",    // HID:34664
        "HID:WIZARDS_HID_DLGCONVERT_OPTIONBUTTON4",    // HID:34665
        "HID:WIZARDS_HID_DLGCONVERT_LISTBOX1",    // HID:34666
        "HID:WIZARDS_HID_DLGCONVERT_OBFILE",    // HID:34667
        "HID:WIZARDS_HID_DLGCONVERT_OBDIR",    // HID:34668
        "HID:WIZARDS_HID_DLGCONVERT_COMBOBOX1",    // HID:34669
        "HID:WIZARDS_HID_DLGCONVERT_TBSOURCE",    // HID:34670
        "HID:WIZARDS_HID_DLGCONVERT_CHECKRECURSIVE",    // HID:34671
        "HID:WIZARDS_HID_DLGCONVERT_TBTARGET",    // HID:34672
        "HID:WIZARDS_HID_DLGCONVERT_CBCANCEL",    // HID:34673
        "HID:WIZARDS_HID_DLGCONVERT_CBHELP",    // HID:34674
        "HID:WIZARDS_HID_DLGCONVERT_CBBACK",    // HID:34675
        "HID:WIZARDS_HID_DLGCONVERT_CBGOON",    // HID:34676
        "HID:WIZARDS_HID_DLGCONVERT_CBSOURCEOPEN",    // HID:34677
        "HID:WIZARDS_HID_DLGCONVERT_CBTARGETOPEN",    // HID:34678
        "HID:WIZARDS_HID_DLGCONVERT_CHKPROTECT",    // HID:34679
        "HID:WIZARDS_HID_DLGCONVERT_CHKTEXTDOCUMENTS",    // HID:34680
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGPASSWORD_CMDGOON",    // HID:34690
        "HID:WIZARDS_HID_DLGPASSWORD_CMDCANCEL",    // HID:34691
        "HID:WIZARDS_HID_DLGPASSWORD_CMDHELP",    // HID:34692
        "HID:WIZARDS_HID_DLGPASSWORD_TXTPASSWORD",    // HID:34693
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_DIALOG",    // HID:34700
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_PREVIEW",    // HID:34701
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_OPYEAR",    // HID:34702
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_OPMONTH",    // HID:34703
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_EDYEAR",    // HID:34704
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_EDMONTH",    // HID:34705
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_SPINYEAR",    // HID:34706
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_SPINMONTH",    // HID:34707
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_1_CMBSTATE",    // HID:34708
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_LBOWNDATA",    // HID:34709
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_CMDINSERT",    // HID:34710
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_CMDDELETE",    // HID:34711
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_EDEVENT",    // HID:34712
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_CHKEVENT",    // HID:34713
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_EDEVENTDAY",    // HID:34714
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_SPINEVENTDAY",    // HID:34715
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_EDEVENTMONTH",    // HID:34716
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_SPINEVENTMONTH",    // HID:34717
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_EDEVENTYEAR",    // HID:34718
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_2_SPINEVENTYEAR",    // HID:34719
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_0_CMDOWNDATA",    // HID:34720
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_0_CMDCANCEL",    // HID:34721
        "HID:WIZARDS_HID_DLGHOLIDAYCAL_0_CMDOK"    // HID:34722
    };
    private static String array2[] = new String[]
    {
        "HID:WIZARDS_HID_LTRWIZ_OPTBUSINESSLETTER",    // HID:40769
        "HID:WIZARDS_HID_LTRWIZ_OPTPRIVOFFICIALLETTER",    // HID:40770
        "HID:WIZARDS_HID_LTRWIZ_OPTPRIVATELETTER",    // HID:40771
        "HID:WIZARDS_HID_LTRWIZ_LSTBUSINESSSTYLE",    // HID:40772
        "HID:WIZARDS_HID_LTRWIZ_CHKBUSINESSPAPER",    // HID:40773
        "HID:WIZARDS_HID_LTRWIZ_LSTPRIVOFFICIALSTYLE",    // HID:40774
        "HID:WIZARDS_HID_LTRWIZ_LSTPRIVATESTYLE",    // HID:40775
        "HID:WIZARDS_HID_LTRWIZ_CHKPAPERCOMPANYLOGO",    // HID:40776
        "HID:WIZARDS_HID_LTRWIZ_NUMLOGOHEIGHT",    // HID:40777
        "HID:WIZARDS_HID_LTRWIZ_NUMLOGOX",    // HID:40778
        "HID:WIZARDS_HID_LTRWIZ_NUMLOGOWIDTH",    // HID:40779
        "HID:WIZARDS_HID_LTRWIZ_NUMLOGOY",    // HID:40780
        "HID:WIZARDS_HID_LTRWIZ_CHKPAPERCOMPANYADDRESS",    // HID:40781
        "HID:WIZARDS_HID_LTRWIZ_NUMADDRESSHEIGHT",    // HID:40782
        "HID:WIZARDS_HID_LTRWIZ_NUMADDRESSX",    // HID:40783
        "HID:WIZARDS_HID_LTRWIZ_NUMADDRESSWIDTH",    // HID:40784
        "HID:WIZARDS_HID_LTRWIZ_NUMADDRESSY",    // HID:40785
        "HID:WIZARDS_HID_LTRWIZ_CHKCOMPANYRECEIVER",    // HID:40786
        "HID:WIZARDS_HID_LTRWIZ_CHKPAPERFOOTER",    // HID:40787
        "HID:WIZARDS_HID_LTRWIZ_NUMFOOTERHEIGHT",    // HID:40788
        "HID:WIZARDS_HID_LTRWIZ_LSTLETTERNORM",    // HID:40789
        "HID:WIZARDS_HID_LTRWIZ_CHKUSELOGO",    // HID:40790
        "HID:WIZARDS_HID_LTRWIZ_CHKUSEADDRESSRECEIVER",    // HID:40791
        "HID:WIZARDS_HID_LTRWIZ_CHKUSESIGNS",    // HID:40792
        "HID:WIZARDS_HID_LTRWIZ_CHKUSESUBJECT",    // HID:40793
        "HID:WIZARDS_HID_LTRWIZ_CHKUSESALUTATION",    // HID:40794
        "HID:WIZARDS_HID_LTRWIZ_LSTSALUTATION",    // HID:40795
        "HID:WIZARDS_HID_LTRWIZ_CHKUSEBENDMARKS",    // HID:40796
        "HID:WIZARDS_HID_LTRWIZ_CHKUSEGREETING",    // HID:40797
        "HID:WIZARDS_HID_LTRWIZ_LSTGREETING",    // HID:40798
        "HID:WIZARDS_HID_LTRWIZ_CHKUSEFOOTER",    // HID:40799
        "HID:WIZARDS_HID_LTRWIZ_OPTSENDERPLACEHOLDER",    // HID:40800
        "HID:WIZARDS_HID_LTRWIZ_OPTSENDERDEFINE",    // HID:40801
        "HID:WIZARDS_HID_LTRWIZ_TXTSENDERNAME",    // HID:40802
        "HID:WIZARDS_HID_LTRWIZ_TXTSENDERSTREET",    // HID:40803
        "HID:WIZARDS_HID_LTRWIZ_TXTSENDERPOSTCODE",    // HID:40804
        "HID:WIZARDS_HID_LTRWIZ_TXTSENDERSTATE_TEXT",    // HID:40805
        "HID:WIZARDS_HID_LTRWIZ_TXTSENDERCITY",    // HID:40806
        "HID:WIZARDS_HID_LTRWIZ_OPTRECEIVERPLACEHOLDER",    // HID:40807
        "HID:WIZARDS_HID_LTRWIZ_OPTRECEIVERDATABASE",    // HID:40808
        "HID:WIZARDS_HID_LTRWIZ_TXTFOOTER",    // HID:40809
        "HID:WIZARDS_HID_LTRWIZ_CHKFOOTERNEXTPAGES",    // HID:40810
        "HID:WIZARDS_HID_LTRWIZ_CHKFOOTERPAGENUMBERS",    // HID:40811
        "HID:WIZARDS_HID_LTRWIZ_TXTTEMPLATENAME",    // HID:40812
        "HID:WIZARDS_HID_LTRWIZ_OPTCREATELETTER",    // HID:40813
        "HID:WIZARDS_HID_LTRWIZ_OPTMAKECHANGES",    // HID:40814
        "HID:WIZARDS_HID_LTRWIZ_TXTPATH",    // HID:40815
        "HID:WIZARDS_HID_LTRWIZ_CMDPATH",    // HID:40816
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_LTRWIZARD",    // HID:40820
        "HID:WIZARDS_HID_LTRWIZARD_HELP",    // HID:40821
        "HID:WIZARDS_HID_LTRWIZARD_BACK",    // HID:40822
        "HID:WIZARDS_HID_LTRWIZARD_NEXT",    // HID:40823
        "HID:WIZARDS_HID_LTRWIZARD_CREATE",    // HID:40824
        "HID:WIZARDS_HID_LTRWIZARD_CANCEL",    // HID:40825
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_LSTTABLES",    // HID:40850
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFIELDS",    // HID:40851
        "HID:WIZARDS_HID_QUERYWIZARD_CMDMOVESELECTED",    // HID:40852
        "HID:WIZARDS_HID_QUERYWIZARD_CMDMOVEALL",    // HID:40853
        "HID:WIZARDS_HID_QUERYWIZARD_CMDREMOVESELECTED",    // HID:40854
        "HID:WIZARDS_HID_QUERYWIZARD_CMDREMOVEALL",    // HID:40855
        "HID:WIZARDS_HID_QUERYWIZARD_LSTSELFIELDS",    // HID:40856
        "HID:WIZARDS_HID_QUERYWIZARD_CMDMOVEUP",    // HID:40857
        "HID:WIZARDS_HID_QUERYWIZARD_CMDMOVEDOWN",    // HID:40858
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_SORT1",    // HID:40865
        "HID:WIZARDS_HID_QUERYWIZARD_OPTASCEND1",    // HID:40866
        "HID:WIZARDS_HID_QUERYWIZARD_OPTDESCEND1",    // HID:40867
        "HID:WIZARDS_HID_QUERYWIZARD_SORT2",    // HID:40868
        "HID:WIZARDS_HID_QUERYWIZARD_OPTASCEND2",    // HID:40869
        "HID:WIZARDS_HID_QUERYWIZARD_OPTDESCEND2",    // HID:40870
        "HID:WIZARDS_HID_QUERYWIZARD_SORT3",    // HID:40871
        "HID:WIZARDS_HID_QUERYWIZARD_OPTASCEND3",    // HID:40872
        "HID:WIZARDS_HID_QUERYWIZARD_OPTDESCEND3",    // HID:40873
        "HID:WIZARDS_HID_QUERYWIZARD_SORT4",    // HID:40874
        "HID:WIZARDS_HID_QUERYWIZARD_OPTASCEND4",    // HID:40875
        "HID:WIZARDS_HID_QUERYWIZARD_OPTDESCEND4",    // HID:40876
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_OPTMATCHALL",    // HID:40878
        "HID:WIZARDS_HID_QUERYWIZARD_OPTMATCHANY",    // HID:40879
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFIELDNAME_1",    // HID:40880
        "HID:WIZARDS_HID_QUERYWIZARD_LSTOPERATOR_1",    // HID:40881
        "HID:WIZARDS_HID_QUERYWIZARD_TXTVALUE_1",    // HID:40882
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFIELDNAME_2",    // HID:40883
        "HID:WIZARDS_HID_QUERYWIZARD_LSTOPERATOR_2",    // HID:40884
        "HID:WIZARDS_HID_QUERYWIZARD_TXTVALUE_2",    // HID:40885
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFIELDNAME_3",    // HID:40886
        "HID:WIZARDS_HID_QUERYWIZARD_LSTOPERATOR_3",    // HID:40887
        "HID:WIZARDS_HID_QUERYWIZARD_TXTVALUE_3",    // HID:40888
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_OPTAGGREGATEDETAILQUERY",    // HID:40895
        "HID:WIZARDS_HID_QUERYWIZARD_OPTAGGREGATESUMMARYQUERY",    // HID:40896
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFUNCTION_1",    // HID:40897
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFIELDS_1",    // HID:40898
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFUNCTION_2",    // HID:40899
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFIELDS_2",    // HID:40900
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFUNCTION_3",    // HID:40901
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFIELDS_3",    // HID:40902
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFUNCTION_4",    // HID:40903
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFIELDS_4",    // HID:40904
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFUNCTION_5",    // HID:40905
        "HID:WIZARDS_HID_QUERYWIZARD_LSTAGGREGATEFIELDS_5",    // HID:40906
        "HID:WIZARDS_HID_QUERYWIZARD_BTNAGGREGATEPLUS",    // HID:40907
        "HID:WIZARDS_HID_QUERYWIZARD_BTNAGGREGATEMINUS",    // HID:40908
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTERFIELDS",    // HID:40915
        "HID:WIZARDS_HID_QUERYWIZARD_CMDFILTERMOVESELECTED",    // HID:40916
        "HID:WIZARDS_HID_QUERYWIZARD_CMDFILTERREMOVESELECTED",    // HID:40917
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTERSELFIELDS",    // HID:40918
        "HID:WIZARDS_HID_QUERYWIZARD_CMDFILTERMOVEUP",    // HID:40919
        "HID:WIZARDS_HID_QUERYWIZARD_CMDFILTERMOVEDOWN",    // HID:40920
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_OPTGROUPMATCHALL",    // HID:40923
        "HID:WIZARDS_HID_QUERYWIZARD_OPTGROUPMATCHANY",    // HID:40924
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTERFIELDNAME_1",    // HID:40925
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTEROPERATOR_1",    // HID:40926
        "HID:WIZARDS_HID_QUERYWIZARD_TXTFILTERVALUE_1",    // HID:40927
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTERFIELDNAME_2",    // HID:40928
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTEROPERATOR_2",    // HID:40929
        "HID:WIZARDS_HID_QUERYWIZARD_TXTFILTERVALUE_2",    // HID:40930
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTERFIELDNAME_3",    // HID:40931
        "HID:WIZARDS_HID_QUERYWIZARD_LSTFILTEROPERATOR_3",    // HID:40932
        "HID:WIZARDS_HID_QUERYWIZARD_TXTFILTERVALUE_3",    // HID:40933
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_1",    // HID:40940
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_2",    // HID:40941
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_3",    // HID:40942
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_4",    // HID:40943
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_5",    // HID:40944
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_6",    // HID:40945
        "HID:WIZARDS_HID_QUERYWIZARD_TXTTITLE_7",    // HID:40946
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_TXTQUERYTITLE",    // HID:40955
        "HID:WIZARDS_HID_QUERYWIZARD_OPTDISPLAYQUERY",    // HID:40956
        "HID:WIZARDS_HID_QUERYWIZARD_OPTMODIFYQUERY",    // HID:40957
        "HID:WIZARDS_HID_QUERYWIZARD_TXTSUMMARY",    // HID:40958
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD",    // HID:40970
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_QUERYWIZARD_BACK",    // HID:40972
        "HID:WIZARDS_HID_QUERYWIZARD_NEXT",    // HID:40973
        "HID:WIZARDS_HID_QUERYWIZARD_CREATE",    // HID:40974
        "HID:WIZARDS_HID_QUERYWIZARD_CANCEL",    // HID:40975
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_IS",    // HID:41000
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_IS_BTN_NONE",    // HID:41002
        "HID:WIZARDS_HID_IS_BTN_OK",    // HID:41003
        "HID:WIZARDS_HID_IS_BTN_CANCEL",    // HID:41004
        "HID:WIZARDS_HID_IS_BTN_IMG1",    // HID:41005
        "HID:WIZARDS_HID_IS_BTN_IMG2",    // HID:41006
        "HID:WIZARDS_HID_IS_BTN_IMG3",    // HID:41007
        "HID:WIZARDS_HID_IS_BTN_IMG4",    // HID:41008
        "HID:WIZARDS_HID_IS_BTN_IMG5",    // HID:41009
        "HID:WIZARDS_HID_IS_BTN_IMG6",    // HID:41010
        "HID:WIZARDS_HID_IS_BTN_IMG7",    // HID:41011
        "HID:WIZARDS_HID_IS_BTN_IMG8",    // HID:41012
        "HID:WIZARDS_HID_IS_BTN_IMG9",    // HID:41013
        "HID:WIZARDS_HID_IS_BTN_IMG10",    // HID:41014
        "HID:WIZARDS_HID_IS_BTN_IMG11",    // HID:41015
        "HID:WIZARDS_HID_IS_BTN_IMG12",    // HID:41016
        "HID:WIZARDS_HID_IS_BTN_IMG13",    // HID:41017
        "HID:WIZARDS_HID_IS_BTN_IMG14",    // HID:41018
        "HID:WIZARDS_HID_IS_BTN_IMG15",    // HID:41019
        "HID:WIZARDS_HID_IS_BTN_IMG16",    // HID:41020
        "HID:WIZARDS_HID_IS_BTN_IMG17",    // HID:41021
        "HID:WIZARDS_HID_IS_BTN_IMG18",    // HID:41022
        "HID:WIZARDS_HID_IS_BTN_IMG19",    // HID:41023
        "HID:WIZARDS_HID_IS_BTN_IMG20",    // HID:41024
        "HID:WIZARDS_HID_IS_BTN_IMG21",    // HID:41025
        "HID:WIZARDS_HID_IS_BTN_IMG22",    // HID:41026
        "HID:WIZARDS_HID_IS_BTN_IMG23",    // HID:41027
        "HID:WIZARDS_HID_IS_BTN_IMG24",    // HID:41028
        "HID:WIZARDS_HID_IS_BTN_IMG25",    // HID:41029
        "HID:WIZARDS_HID_IS_BTN_IMG26",    // HID:41030
        "HID:WIZARDS_HID_IS_BTN_IMG27",    // HID:41031
        "HID:WIZARDS_HID_IS_BTN_IMG28",    // HID:41032
        "HID:WIZARDS_HID_IS_BTN_IMG29",    // HID:41033
        "HID:WIZARDS_HID_IS_BTN_IMG30",    // HID:41034
        "HID:WIZARDS_HID_IS_BTN_IMG31",    // HID:41035
        "HID:WIZARDS_HID_IS_BTN_IMG32",    // HID:41036
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_FTP",    // HID:41040
        "HID:WIZARDS_HID_FTP_SERVER",    // HID:41041
        "HID:WIZARDS_HID_FTP_USERNAME",    // HID:41042
        "HID:WIZARDS_HID_FTP_PASS",    // HID:41043
        "HID:WIZARDS_HID_FTP_TEST",    // HID:41044
        "HID:WIZARDS_HID_FTP_TXT_PATH",    // HID:41045
        "HID:WIZARDS_HID_FTP_BTN_PATH",    // HID:41046
        "HID:WIZARDS_HID_FTP_OK",    // HID:41047
        "HID:WIZARDS_HID_FTP_CANCEL",    // HID:41048
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_AGWIZ",    // HID:41051
        "HID:WIZARDS_HID_AGWIZ_HELP",    // HID:41052
        "HID:WIZARDS_HID_AGWIZ_NEXT",    // HID:41053
        "HID:WIZARDS_HID_AGWIZ_PREV",    // HID:41054
        "HID:WIZARDS_HID_AGWIZ_CREATE",    // HID:41055
        "HID:WIZARDS_HID_AGWIZ_CANCEL",    // HID:41056
        "HID:WIZARDS_HID_AGWIZ_1_LIST_PAGEDESIGN",    // HID:41057
        "HID:WIZARDS_HID_AGWIZ_1_CHK_MINUTES",    // HID:41058
        "HID:WIZARDS_HID_AGWIZ_2_TXT_TIME",    // HID:41059
        "HID:WIZARDS_HID_AGWIZ_2_TXT_DATE",    // HID:41060
        "HID:WIZARDS_HID_AGWIZ_2_TXT_TITLE",    // HID:41061
        "HID:WIZARDS_HID_AGWIZ_2_TXT_LOCATION",    // HID:41062
        "HID:WIZARDS_HID_AGWIZ_3_CHK_MEETING_TYPE",    // HID:41063
        "HID:WIZARDS_HID_AGWIZ_3_CHK_READ",    // HID:41064
        "HID:WIZARDS_HID_AGWIZ_3_CHK_BRING",    // HID:41065
        "HID:WIZARDS_HID_AGWIZ_3_CHK_NOTES",    // HID:41066
        "HID:WIZARDS_HID_AGWIZ_4_CHK_CALLED_BY",    // HID:41067
        "HID:WIZARDS_HID_AGWIZ_4_CHK_FACILITATOR",    // HID:41068
        "HID:WIZARDS_HID_AGWIZ_4_CHK_NOTETAKER",    // HID:41069
        "HID:WIZARDS_HID_AGWIZ_4_CHK_TIMEKEEPER",    // HID:41070
        "HID:WIZARDS_HID_AGWIZ_4_CHK_ATTENDEES",    // HID:41071
        "HID:WIZARDS_HID_AGWIZ_4_CHK_OBSERVERS",    // HID:41072
        "HID:WIZARDS_HID_AGWIZ_4_CHK_RESOURCEPERSONS",    // HID:41073
        "HID:WIZARDS_HID_AGWIZ_6_TXT_TEMPLATENAME",    // HID:41074
        "HID:WIZARDS_HID_AGWIZ_6_TXT_TEMPLATEPATH",    // HID:41075
        "HID:WIZARDS_HID_AGWIZ_6_BTN_TEMPLATEPATH",    // HID:41076
        "HID:WIZARDS_HID_AGWIZ_6_OPT_CREATEAGENDA",    // HID:41077
        "HID:WIZARDS_HID_AGWIZ_6_OPT_MAKECHANGES",    // HID:41078
        "HID:WIZARDS_HID_AGWIZ_5_BTN_INSERT",    // HID:41079
        "HID:WIZARDS_HID_AGWIZ_5_BTN_REMOVE",    // HID:41080
        "HID:WIZARDS_HID_AGWIZ_5_BTN_UP",    // HID:41081
        "HID:WIZARDS_HID_AGWIZ_5_BTN_DOWN",    // HID:41082
        "HID:WIZARDS_HID_AGWIZ_5_SCROLL_BAR",    // HID:41083
        "HID:WIZARDS_HID_AGWIZ_5_TXT_TOPIC_1",    // HID:41084
        "HID:WIZARDS_HID_AGWIZ_5_TXT_RESPONSIBLE_1",    // HID:41085
        "HID:WIZARDS_HID_AGWIZ_5_TXT_MINUTES_1",    // HID:41086
        "HID:WIZARDS_HID_AGWIZ_5_TXT_TOPIC_2",    // HID:41087
        "HID:WIZARDS_HID_AGWIZ_5_TXT_RESPONSIBLE_2",    // HID:41088
        "HID:WIZARDS_HID_AGWIZ_5_TXT_MINUTES_2",    // HID:41089
        "HID:WIZARDS_HID_AGWIZ_5_TXT_TOPIC_3",    // HID:41090
        "HID:WIZARDS_HID_AGWIZ_5_TXT_RESPONSIBLE_3",    // HID:41091
        "HID:WIZARDS_HID_AGWIZ_5_TXT_MINUTES_3",    // HID:41092
        "HID:WIZARDS_HID_AGWIZ_5_TXT_TOPIC_4",    // HID:41093
        "HID:WIZARDS_HID_AGWIZ_5_TXT_RESPONSIBLE_4",    // HID:41094
        "HID:WIZARDS_HID_AGWIZ_5_TXT_MINUTES_4",    // HID:41095
        "HID:WIZARDS_HID_AGWIZ_5_TXT_TOPIC_5",    // HID:41096
        "HID:WIZARDS_HID_AGWIZ_5_TXT_RESPONSIBLE_5",    // HID:41097
        "HID:WIZARDS_HID_AGWIZ_5_TXT_MINUTES_5",    // HID:41098
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_FAXWIZ_OPTBUSINESSFAX",    // HID:41120
        "HID:WIZARDS_HID_FAXWIZ_LSTBUSINESSSTYLE",    // HID:41121
        "HID:WIZARDS_HID_FAXWIZ_OPTPRIVATEFAX",    // HID:41122
        "HID:WIZARDS_HID_LSTPRIVATESTYLE",    // HID:41123
        "HID:WIZARDS_HID_IMAGECONTROL3",    // HID:41124
        "HID:WIZARDS_HID_CHKUSELOGO",    // HID:41125
        "HID:WIZARDS_HID_CHKUSEDATE",    // HID:41126
        "HID:WIZARDS_HID_CHKUSECOMMUNICATIONTYPE",    // HID:41127
        "HID:WIZARDS_HID_LSTCOMMUNICATIONTYPE",    // HID:41128
        "HID:WIZARDS_HID_CHKUSESUBJECT",    // HID:41129
        "HID:WIZARDS_HID_CHKUSESALUTATION",    // HID:41130
        "HID:WIZARDS_HID_LSTSALUTATION",    // HID:41131
        "HID:WIZARDS_HID_CHKUSEGREETING",    // HID:41132
        "HID:WIZARDS_HID_LSTGREETING",    // HID:41133
        "HID:WIZARDS_HID_CHKUSEFOOTER",    // HID:41134
        "HID:WIZARDS_HID_OPTSENDERPLACEHOLDER",    // HID:41135
        "HID:WIZARDS_HID_OPTSENDERDEFINE",    // HID:41136
        "HID:WIZARDS_HID_TXTSENDERNAME",    // HID:41137
        "HID:WIZARDS_HID_TXTSENDERSTREET",    // HID:41138
        "HID:WIZARDS_HID_TXTSENDERPOSTCODE",    // HID:41139
        "HID:WIZARDS_HID_TXTSENDERSTATE",    // HID:41140
        "HID:WIZARDS_HID_TXTSENDERCITY",    // HID:41141
        "HID:WIZARDS_HID_TXTSENDERFAX",    // HID:41142
        "HID:WIZARDS_HID_OPTRECEIVERPLACEHOLDER",    // HID:41143
        "HID:WIZARDS_HID_OPTRECEIVERDATABASE",    // HID:41144
        "HID:WIZARDS_HID_TXTFOOTER",    // HID:41145
        "HID:WIZARDS_HID_CHKFOOTERNEXTPAGES",    // HID:41146
        "HID:WIZARDS_HID_CHKFOOTERPAGENUMBERS",    // HID:41147
        "HID:WIZARDS_HID_TXTTEMPLATENAME",    // HID:41148
        "HID:WIZARDS_HID_FILETEMPLATEPATH",    // HID:41149
        "HID:WIZARDS_HID_OPTCREATEFAX",    // HID:41150
        "HID:WIZARDS_HID_OPTMAKECHANGES",    // HID:41151
        "HID:WIZARDS_HID_IMAGECONTROL2",    // HID:41152
        "HID:WIZARDS_HID_FAXWIZ_TXTPATH",    // HID:41153
        "HID:WIZARDS_HID_FAXWIZ_CMDPATH",    // HID:41154
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_FAXWIZARD",    // HID:41180
        "HID:WIZARDS_HID_FAXWIZARD_HELP",    // HID:41181
        "HID:WIZARDS_HID_FAXWIZARD_BACK",    // HID:41182
        "HID:WIZARDS_HID_FAXWIZARD_NEXT",    // HID:41183
        "HID:WIZARDS_HID_FAXWIZARD_CREATE",    // HID:41184
        "HID:WIZARDS_HID_FAXWIZARD_CANCEL",    // HID:41185
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGTABLE_DIALOG",    // HID:41200
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGTABLE_CMDPREV",    // HID:41202
        "HID:WIZARDS_HID_DLGTABLE_CMDNEXT",    // HID:41203
        "HID:WIZARDS_HID_DLGTABLE_CMDFINISH",    // HID:41204
        "HID:WIZARDS_HID_DLGTABLE_CMDCANCEL",    // HID:41205
        "HID:WIZARDS_HID_DLGTABLE_OPTBUSINESS",    // HID:41206
        "HID:WIZARDS_HID_DLGTABLE_OPTPRIVATE",    // HID:41207
        "HID:WIZARDS_HID_DLGTABLE_LBTABLES",    // HID:41208
        "HID:WIZARDS_HID_DLGTABLE_FIELDSAVAILABLE",    // HID:41209
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVESELECTED",    // HID:41210
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEALL",    // HID:41211
        "HID:WIZARDS_HID_DLGTABLE_CMDREMOVESELECTED",    // HID:41212
        "HID:WIZARDS_HID_DLGTABLE_CMDREMOVEALL",    // HID:41213
        "HID:WIZARDS_HID_DLGTABLE_FIELDSSELECTED",    // HID:41214
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEUP",    // HID:41215
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEDOWN",    // HID:41216
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        PropertyNames.EMPTY_STRING,
        "HID:WIZARDS_HID_DLGTABLE_LB_SELFIELDNAMES",    // HID:41220
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEFIELDUP",    // HID:41221
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEFIELDDOWN",    // HID:41222
        "HID:WIZARDS_HID_DLGTABLE_CMDMINUS",    // HID:41223
        "HID:WIZARDS_HID_DLGTABLE_CMDPLUS",    // HID:41224
        "HID:WIZARDS_HID_DLGTABLE_COLNAME",    // HID:41225
        "HID:WIZARDS_HID_DLGTABLE_COLMODIFIER",    // HID:41226
        "HID:WIZARDS_HID_DLGTABLE_CHK_USEPRIMEKEY",    // HID:41227
        "HID:WIZARDS_HID_DLGTABLE_OPT_PK_AUTOMATIC",    // HID:41228
        "HID:WIZARDS_HID_DLGTABLE_CK_PK_AUTOVALUE_AUTOMATIC",    // HID:41229
        "HID:WIZARDS_HID_DLGTABLE_OPT_PK_SINGLE",    // HID:41230
        "HID:WIZARDS_HID_DLGTABLE_LB_PK_FIELDNAME",    // HID:41231
        "HID:WIZARDS_HID_DLGTABLE_CK_PK_AUTOVALUE",    // HID:41232
        "HID:WIZARDS_HID_DLGTABLE_OPT_PK_SEVERAL",    // HID:41233
        "HID:WIZARDS_HID_DLGTABLE_FIELDS_PK_AVAILABLE",    // HID:41234
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVE_PK_SELECTED",    // HID:41235
        "HID:WIZARDS_HID_DLGTABLE_CMDREMOVE_PK_SELECTED",    // HID:41236
        "HID:WIZARDS_HID_DLGTABLE_FIELDS_PK_SELECTED",    // HID:41237
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEUP_PK_SELECTED",    // HID:41238
        "HID:WIZARDS_HID_DLGTABLE_CMDMOVEDOWN_PK_SELECTED",    // HID:41239
        "HID:WIZARDS_HID_DLGTABLE_TXT_NAME",    // HID:41240
        "HID:WIZARDS_HID_DLGTABLE_OPT_MODIFYTABLE",    // HID:41241
        "HID:WIZARDS_HID_DLGTABLE_OPT_WORKWITHTABLE",    // HID:41242
        "HID:WIZARDS_HID_DLGTABLE_OPT_STARTFORMWIZARD",    // HID:41243
        "HID:WIZARDS_HID_DLGTABLE_LST_CATALOG",    // HID:41244
        "HID:WIZARDS_HID_DLGTABLE_LST_SCHEMA"    // HID:41245
    };

    public static String getHelpIdString( int nHelpId )
    {
        if ( nHelpId >= 34200 && nHelpId <= 34722 )
            return array1[ nHelpId - 34200 ];
        else if ( nHelpId >= 40769 && nHelpId <= 41245 )
            return array2[ nHelpId - 40769 ];
        else
            return "";
    }
}
