#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# This file incorporates work covered by the following license notice:
#
#   Licensed to the Apache Software Foundation (ASF) under one or more
#   contributor license agreements. See the NOTICE file distributed
#   with this work for additional information regarding copyright
#   ownership. The ASF licenses this file to you under the Apache
#   License, Version 2.0 (the "License"); you may not use this file
#   except in compliance with the License. You may obtain a copy of
#   the License at http://www.apache.org/licenses/LICENSE-2.0 .
#
LSTLOADSETTINGS_ITEM_CHANGED = "sessionSelected" # "lstLoadSettingsItemChanged"
LSTLOADSETTINGS_ITEM_CHANGED = "sessionSelected" # "lstLoadSettingsItemChanged"
BTNLOADSESSION_ACTION_PERFORMED = "loadSession" # "btnLoadSessionActionPerformed"
BTNDELSESSION_ACTION_PERFORMED = "delSession" # "btnDelSessionActionPerformed"
BTNADDDOC_ACTION_PERFORMED = "addDocument" # "btnAddDocActionPerformed"
BTNREMOVEDOC_ACTION_PERFORMED = "removeDocument" # "btnRemoveDocActionPerformed"
BTNDOCUP_ACTION_PERFORMED = "docUp" # "btnDocUpActionPerformed"
BTNDOCDOWN_ACTION_PERFORMED = "docDown" # "btnDocDownActionPerformed"
LSTSTYLES_ITEM_CHANGED = "refreshStylePreview" # "lstStylesItemChanged"
BTNBACKGROUNDS_ACTION_PERFORMED = "chooseBackground" # "btnBackgroundsActionPerformed"
BTNICONSETS_ACTION_PERFORMED = "chooseIconset" # "btnIconSetsActionPerformed"
BTNFAVICON_ACTION_PERFORMED = "chooseFavIcon" # "btnFavIconActionPerformed"
BTNPREVIEW_ACTION_PERFORMED = "documentPreview" # "btnPreviewActionPerformed"
BTNFTP_ACTION_PERFORMED = "setFTPPublish" # "btnFTPActionPerformed"
CHKLOCALDIR_ITEM_CHANGED = "chkLocalDirItemChanged" # "chkLocalDirItemChanged"
CHKZIP_ITEM_CHANGED = "chkZipItemChanged"
CHKSAVESETTINGS_ITEM_CHANGED = "checkPublish" # "chkSaveSettingsItemChanged"
TXTSAVESETTINGS_TEXT_CHANGED = "checkPublish" # "txtSaveSettingsTextChanged"
BTNLOCALDIR_ACTION_PERFORMED = "setPublishLocalDir" # "btnLocalDirActionPerformed"
BTNZIP_ACTION_PERFORMED = "setZipFilename" # "btnZipActionPerformed"
CONFIG_PATH = "/org.openoffice.Office.WebWizard/WebWizard"
CONFIG_READ_PARAM = "cp_"
TASK = "WWIZ"
TASK_PREPARE = "t-prep"
LOCAL_PUBLISHER = "local"
FTP_PUBLISHER = "ftp"
ZIP_PUBLISHER = "zip"
TASK_EXPORT = "t_exp"
TASK_EXPORT_PREPARE = "t_exp_prep"
TASK_EXPORT_DOCUMENTS = "t_exp_docs"
TASK_GENERATE_PREPARE = "t_gen_prep"
TASK_GENERATE_XSL = "t_gen_x"
TASK_GENERATE_COPY = "t_gen_cp"
TASK_PUBLISH_PREPARE = "t_pub_prep"
TASK_FINISH = "t_fin"

'''
when the user adds more than this number
of documents to the list, a status dialog opens.
'''
MIN_ADD_FILES_FOR_DIALOG = 2
