#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# This file incorporates work covered by the following license notice:
#
#   Licensed to the Apache Software Foundation (ASF) under one or more
#   contributor license agreements. See the NOTICE file distributed
#   with this work for additional information regarding copyright
#   ownership. The ASF licenses this file to you under the Apache
#   License, Version 2.0 (the "License"); you may not use this file
#   except in compliance with the License. You may obtain a copy of
#   the License at http://www.apache.org/licenses/LICENSE-2.0 .

# Error IDs for errors that can accure
# in the interaction with the Process class.
class ProcessErrors:

    ERROR_MKDIR = 0
    ERROR_EXPORT = 1
    ERROR_EXPORT_MKDIR = 2
    ERROR_DOC_VALIDATE = 3
    ERROR_EXPORT_IO = 4
    ERROR_EXPORT_SECURITY = 5
    ERROR_GENERATE_XSLT = 6
    ERROR_GENERATE_COPY = 7
    ERROR_PUBLISH = 8
    ERROR_PUBLISH_MEDIA = 9
    ERROR_CLEANUP = 10
